CREATE OR REPLACE PACKAGE BODY xml AS
  parse_error EXCEPTION;
  PRAGMA EXCEPTION_INIT(parse_error,-20100);
  http_proxy_host VARCHAR2(200);
  http_proxy_port VARCHAR2(5) := '80';

  PROCEDURE setHttpProxy(machinename VARCHAR2, port VARCHAR2 := '80') IS
  BEGIN
    http_proxy_host := machinename;
    http_proxy_port := port;
  END;

  -- Set HTTP Proxy for Java programs in the current session.
  PROCEDURE setProxy IS
  BEGIN
    IF http_proxy_host IS NOT NULL THEN
      http_util.setProxy(http_proxy_host,http_proxy_port);
    END IF;
  END;

  ----------------------------------------------------------------------
  -- Parse functions parse an XML document and return a handle to
  -- the in-memory DOM Document representation of the parsed XML.
  -- Call freeDocument() when you're done using the document returned
  -- by the function.
  ----------------------------------------------------------------------
  FUNCTION parse(xml VARCHAR2) RETURN xmldom.DOMDocument IS
    retDoc xmldom.DOMDocument;
    parser xmlparser.Parser;
  BEGIN
    IF xml IS NULL THEN RETURN NULL; END IF;
    setProxy;
    parser := xmlparser.newParser;
    xmlparser.parseBuffer(parser,xml);
    retDoc := xmlparser.getDocument(parser);
    xmlparser.freeParser(parser);
    RETURN retDoc;
  EXCEPTION
    WHEN parse_error THEN
      xmlparser.freeParser(parser);
      RETURN retdoc;
  END;

  FUNCTION parse(xml BFILE) RETURN xmldom.DOMDocument IS
    retDoc xmldom.DOMDocument;
    parser xmlparser.Parser;
    b BFILE := xml;
    c CLOB;
  BEGIN
    IF xml IS NULL THEN RETURN NULL; END IF;
    setProxy;
    parser := xmlparser.newParser;
    dbms_lob.createtemporary(c,cache=>FALSE);
    dbms_lob.fileOpen(b);
    dbms_lob.loadFromFile(dest_lob => c,
                           src_lob => b,
                           amount  => dbms_lob.getLength(b));
    dbms_lob.fileClose(b);
    xmlparser.parseCLOB(parser,c);
    retDoc := xmlparser.getDocument(parser);
    dbms_lob.freetemporary(c);
    xmlparser.freeParser(parser);
    RETURN retDoc;
  EXCEPTION
    WHEN parse_error THEN
      dbms_lob.freetemporary(c);
      xmlparser.freeParser(parser);
      RETURN retdoc;
  END;

  FUNCTION parse(xml CLOB) RETURN xmldom.DOMDocument IS
    retDoc xmldom.DOMDocument;
    parser xmlparser.Parser;
  BEGIN
    IF xml IS NULL THEN RETURN NULL; END IF;
    setProxy;
    parser := xmlparser.newParser;
    xmlparser.parseCLOB(parser,xml);
    retDoc := xmlparser.getDocument(parser);
    xmlparser.freeParser(parser);
    RETURN retDoc;
  EXCEPTION
    WHEN parse_error THEN
      xmlparser.freeParser(parser);
      RETURN retdoc;
  END;

  FUNCTION parseURL(url VARCHAR2) RETURN xmldom.DOMDocument IS
    xmldoc xmldom.DOMDocument;
  BEGIN
    IF url IS NULL THEN RETURN NULL; END IF;
    setProxy;
    RETURN xmlparser.parse(url);
  END;

  -- Free the Java objects associated with an in-memory DOM tree
  PROCEDURE freeDocument(doc xmldom.DOMDocument) IS
  BEGIN
    xmldom.freeDocument(doc);
  END;

END;
