CREATE OR REPLACE PACKAGE BODY xml_claim AS

  PROCEDURE store( doc xmldom.DOMDocument ) IS
    claim   ins_claim%ROWTYPE;
    payment ins_claim_payment%ROWTYPE;
    damrep     CLOB;
    damrepText VARCHAR2(32767);
    payments xmldom.DOMNodeList;
    curNode  xmldom.DOMNode;
  BEGIN
    -- Use xpath.valueOf to retrieve values for Insert
    claim.claimid := xpath.valueOf(doc,'/Claim/ClaimId');
    claim.policy  := xpath.valueOf(doc,'/Claim/Policy');

    -- Use xpath.extract to extract matching XML fragment
    damrepText    := xpath.extract(doc,'/Claim/DamageReport');

    INSERT INTO ins_claim(claimid,policy,damagereport)
         VALUES (claim.claimid, claim.policy, empty_clob())
    RETURNING damagereport INTO damrep;

    -- Write the damagereport xml document fragment into the CLOB column
    dbms_lob.writeappend(damrep,LENGTH(damrepText),damrepText);

    -- Process all the <Payment> elements in the <Settlements> section
    payments := xpath.selectNodes(doc,'/Claim/Settlements/Payment');
    FOR j IN 1..xmldom.getLength(payments) LOOP

      -- Recall that the Node List is zero-based!
      curNode := xmldom.item(payments, j-1);

      -- Use xpath.valueOf to retrieve the values for Insert
      -- Note that Amount is the value of the current "Payment" element
      payment.amount   := xpath.valueOf(curNode,'.');
      payment.approver := xpath.valueOf(curNode,'@Approver');

      INSERT INTO ins_claim_payment(claimid, amount, approver)
           VALUES(claim.claimid,payment.amount,payment.approver);

    END LOOP;
  END;

  PROCEDURE store( claimdoc CLOB ) IS
    xmldoc xmldom.DOMDocument;
  BEGIN
    -- If we're called with a CLOB, parse it and pass it to the other store()
    xmldoc := xml.parse(claimdoc);
    store(xmldoc);
    xml.freeDocument(xmldoc);
  EXCEPTION
    WHEN OTHERS THEN xml.freeDocument(xmldoc); RAISE;
  END;

END;
